# -*- coding: utf-8 -*-
import os
import re
import sys

try:
    from skplatform._version import __version__
except:
    __version__ = '??'

# Minimum version, enforced by sphinx
needs_sphinx = '3.2.0'

extensions = ['sphinx.ext.autodoc',
              'sphinx.ext.autosummary',
              'sphinx.ext.mathjax',
              'sphinx.ext.viewcode',
              'sphinx.ext.intersphinx',
              'numpydoc',
              'sphinxcontrib.bibtex',
              'sphinx.ext.todo'
              ]

if sys.platform == "win32":
    try:
        import sphinx2word
        extensions.append('sphinx2word')
    except:
        print("Error importing the sphinx2word extension. Try: c:> pip install sphinx2word -f https://arg/usask/ca/wheels")

bibtex_bibfiles = []
bibtex_reference_style = 'author_year'

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']
numfig = True
numfig_secnum_depth = 1

# The suffix(es) of source filenames.
# You can specify multiple suffix as a list of string:
#
# source_suffix = ['.rst', '.md']
source_suffix = '.rst'

# The master toctree document.
master_doc = 'index'

# General information about the project.
project = 'skplatform'
copyright = '2025, ARG University Of Saskatchewan'
author = 'USASK-ARG'

# The short X.Y version.
version = __version__
# "0.1.0"
# The full version, including alpha/beta/rc tags.
release = ''

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = 'en'

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This patterns also effect to html_static_path and html_extra_path
exclude_patterns = ['_build', '*deprecated', 'Thumbs.db', '.DS_Store']

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'

# If true, `todo` and `todoList` produce output, else they produce nothing.
todo_include_todos = False

# If true, '()' will be appended to :func: etc. cross-reference text.
add_function_parentheses = False

# If true, the current module name will be prepended to all description
# unit titles (such as .. function::).
add_module_names = False

language = 'en'
html_theme = 'pydata_sphinx_theme'
logo_name = 'skproject-logo.png'
html_logo ='_static/{}'.format(logo_name)
html_static_path = ["_static"]
html_favicon = "_static/usask-arg_icon_32.ico"
html_theme_options = {
   "icon_links": [ {"name": "Github",
                    "url": "https://github.com/usask-arg/skplatform",
                    "icon": "fas fa-code-branch"
                    },
                   ],
    "logo": { "image_light": logo_name, "image_dark": logo_name, 'text':'{}'.format(version)}
}

# -- Options for LaTeX output ---------------------------------------------
latex_elements = {
    # The paper size ('letterpaper' or 'a4paper').
    #
    # 'papersize': 'letterpaper',

    # The font size ('10pt', '11pt' or '12pt').
    #
    # 'pointsize': '10pt',

    # Additional stuff for the LaTeX preamble.
    #
    # 'preamble': '',

    # Latex figure (float) alignment
    #
    # 'figure_align': 'htbp',
}

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title,
#  author, documentclass [howto, manual, or own class]).
latex_documents = [
    (master_doc, 'skplatform.tex', 'skplatform', 'ARG', 'manual'),
]

# -- Options for manual page output ---------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    (master_doc, 'skplatform', 'skplatform', [author], 1)
]

# -- Options for Texinfo output -------------------------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [
    (master_doc, 'skplatform', 'skplatform', author, 'skplatform', 'The skplatform project.',
     'Miscellaneous'),
]

rst_prolog = """
    .. include:: <s5defs.txt>
"""

html_css_files = [
    'sksoftware.css'
]